/************************************************************************
 *   Internet Relay Chat Daemon, include/msg.h
 *   Copyright (C) 1990 Jarkko Oikarinen and
 *                      University of Oulu, Computing Center
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 1, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *   $Id: msg.h,v 1.1.1.1.6.1.2.15 2000/11/23 19:08:02 codemastr Exp $
 */

#ifndef	__msg_include__
#define __msg_include__

/*
 * The tokens are in the ascii character range of 33-127, and we start
 * from 33 and just move up.  It would be nice to match then up so they
 * are slightly related to their string counterpart, but that makes it
 * too confusing when we want to add another one and need to make sure
 * we're not using one already used. -Cabal95
 *
 * As long as the #defines are kept statically placed, it will be fine.
 * We don't care/worry about the msgtab[] since it can be dynamic, but
 * the tokens it uses will still be static according to the messages
 * they represent.  In other words leave the #defines in order, if you're
 * going to add something, PUT IT AT THE END.  Do not even look for an
 * open spot somewhere, as that may lead to one type of message being
 * sent by server A to server B, but server B thinks its something else.
 * Remember, skip the : since its got a special use, and I skip the \ too
 * since it _may_ cause problems, but not sure.  -Cabal95
 * I'm skipping A and a as well, because some clients and scripts use
 * these to test if the server has already processed whole queue.
 * Since the client could request this protocol withhout the script
 * knowing it, I'm considering that reserved, and TRACE/A is now 'b'.
 * The normal msgtab should probably process this as special. -Donwulff
 */

/*	12/05/1999 - I was wrong - I didnt see the token[2] in struct Message
	okie 60*60 commands more :P - Sowwy!!! -sts
	
 */

#define MSG_PRIVATE	"PRIVMSG"
#define TOK_PRIVATE	"!"
#define MSG_WHO		"WHO"
#define TOK_WHO		"\""
#define MSG_WHOIS	"WHOIS"
#define TOK_WHOIS	"#"
#define MSG_WHOWAS	"WHOWAS"
#define TOK_WHOWAS	"$"
#define MSG_USER	"USER"
#define TOK_USER	"%"
#define MSG_NICK	"NICK"
#define TOK_NICK	"&"
#define MSG_SERVER	"SERVER"
#define TOK_SERVER	"'"
#define MSG_LIST	"LIST"
#define TOK_LIST	"("
#define MSG_TOPIC	"TOPIC"
#define TOK_TOPIC	")"
#define MSG_INVITE	"INVITE"
#define TOK_INVITE	"*"
#define MSG_VERSION	"VERSION"
#define TOK_VERSION	"+"
#define MSG_QUIT	"QUIT"
#define TOK_QUIT	","
#define MSG_SQUIT	"SQUIT"
#define TOK_SQUIT	"-"
#define MSG_KILL	"KILL"
#define TOK_KILL	"."
#define MSG_INFO	"INFO"
#define TOK_INFO	"/"
#define MSG_LINKS	"LINKS"
#define TOK_LINKS	"0"
#define MSG_LINKINFO	"LINKINFO"
#define TOK_LINKINFO	"LI"
#define MSG_SUMMON	"SUMMON"
#define TOK_SUMMON	"1"
#define MSG_STATS	"STATS"
#define TOK_STATS	"2"
#define MSG_USERS	"USERS"
#define TOK_USERS	"3"
#define MSG_HELP	"HELP"
#define MSG_HELPOP	"HELPOP"
#define TOK_HELP	"4"
#define MSG_ERROR	"ERROR"
#define TOK_ERROR	"5"
#define MSG_AWAY	"AWAY"
#define TOK_AWAY	"6"
#define MSG_CONNECT	"CONNECT"
#define TOK_CONNECT	"7"
#define MSG_PING	"PING"
#define TOK_PING	"8"
#define MSG_PONG	"PONG"
#define TOK_PONG	"9"
#define MSG_OPER	"OPER"
#define TOK_OPER	";"
#define MSG_PASS	"PASS"
#define TOK_PASS	"<"
#define MSG_WALLOPS	"WALLOPS"
#define TOK_WALLOPS	"="
#define MSG_TIME	"TIME"
#define TOK_TIME	">"
#define MSG_NAMES	"NAMES"
#define TOK_NAMES	"?"
#define MSG_ADMIN	"ADMIN"
#define TOK_ADMIN	"@"
#define MSG_NOTICE	"NOTICE"
#define TOK_NOTICE	"B"
#define MSG_JOIN	"JOIN"
#define TOK_JOIN	"C"
#define MSG_PART	"PART"
#define TOK_PART	"D"
#define MSG_LUSERS	"LUSERS"
#define TOK_LUSERS	"E"
#define MSG_MOTD	"MOTD"
#define TOK_MOTD	"F"
#define MSG_MODE	"MODE"
#define TOK_MODE	"G"
#define MSG_KICK	"KICK"
#define TOK_KICK	"H"
#define MSG_SERVICE	"SERVICE"
#define TOK_SERVICE	"I"
#define MSG_USERHOST	"USERHOST"
#define TOK_USERHOST	"J"
#define MSG_ISON	"ISON"
#define TOK_ISON	"K"
#define	MSG_REHASH	"REHASH"
#define TOK_REHASH	"O"
#define	MSG_RESTART	"RESTART"
#define TOK_RESTART	"P"
#define	MSG_CLOSE	"CLOSE"
#define TOK_CLOSE	"Q"
#define	MSG_DIE		"DIE"
#define TOK_DIE		"R"
#define	MSG_HASH	"HASH"
#define TOK_HASH	"S"
#define	MSG_DNS		"DNS"
#define TOK_DNS		"T"
#define MSG_SILENCE	"SILENCE"
#define TOK_SILENCE	"U"
#define MSG_AKILL	"AKILL"
#define TOK_AKILL	"V"
#define MSG_KLINE	"KLINE"
#define TOK_KLINE	"W"
#define MSG_UNKLINE     "UNKLINE"
#define TOK_UNKLINE	"X"
#define MSG_RAKILL	"RAKILL"
#define TOK_RAKILL	"Y"
#define MSG_GNOTICE	"GNOTICE"
#define TOK_GNOTICE	"Z"
#define MSG_GOPER	"GOPER"
#define TOK_GOPER	"["
#define MSG_GLOBOPS	"GLOBOPS"
#define TOK_GLOBOPS	"]"
#define MSG_LOCOPS	"LOCOPS"
#define TOK_LOCOPS	"^"
#define MSG_HELPERS	"HELPERS"
#define TOK_HELPERS	"HPS"
#define MSG_VIRUS	"VIRUS"
#define TOK_VIRUS	"VIR"
#define MSG_PROTOCTL	"PROTOCTL"
#define TOK_PROTOCTL	"_"
#define MSG_WATCH	"WATCH"
#define TOK_WATCH	"`"
#define MSG_TRACE	"TRACE"
#define TOK_TRACE	"b"
#define MSG_SQLINE	"SQLINE"
#define TOK_SQLINE	"c"
#define MSG_UNSQLINE	"UNSQLINE"
#define TOK_UNSQLINE	"d"
#define MSG_SVSNICK	"SVSNICK"
#define TOK_SVSNICK	"e"
#define MSG_SVSNOOP	"SVSNOOP"
#define TOK_SVSNOOP	"f"
#define MSG_IDENTIFY	"IDENTIFY"
#define TOK_IDENTIFY	"g"
#define MSG_AUTH	"AUTH"
#define TOK_AUTH 	"ga"
#define MSG_SVSKILL	"SVSKILL"
#define TOK_SVSKILL	"h"
#define MSG_NICKSERV	"NICKSERV"
#define MSG_NS		"NS"
#define TOK_NICKSERV	"i"
#define MSG_CHANSERV	"CHANSERV"
#define MSG_CS		"CS"
#define TOK_CHANSERV	"j"
#define MSG_BACKUP	"BACKUP"
#define MSG_BU		"BU"
#define TOK_BACKUP	"k"
#define MSG_MEMOSERV	"MEMOSERV"
#define MSG_MS		"MS"
#define TOK_MEMOSERV	"l"
#define MSG_SERVICES	"SERVICES"
#define MSG_SS          "SS"
#define TOK_SERVICES	"m"
#define MSG_SVSMODE	"SVSMODE"
#define TOK_SVSMODE	"n"
#define MSG_SAMODE	"SAMODE"
#define TOK_SAMODE	"o"
#define MSG_CHATOPS	"CHATOPS"
#define TOK_CHATOPS	"p"
#define MSG_ZLINE    	"ZLINE"
#define TOK_ZLINE	"q"
#define MSG_UNZLINE  	"UNZLINE"
#define TOK_UNZLINE	"r"
#define MSG_HELPSERV    "HELPSERV"
#define MSG_HS		"HS"
#define TOK_HELPSERV    "s"
#define MSG_RULES       "RULES"
#define TOK_RULES       "t"
#define MSG_MAP         "MAP"
#define TOK_MAP         "u"
#define MSG_SVS2MODE    "SVS2MODE"
#define TOK_SVS2MODE	"v"
#define MSG_DALINFO     "DALINFO"
#define TOK_DALINFO     "w"
#define MSG_MKPASSWD	"MKPASSWD"
#define TOK_MKPASSWD	"y"
#define MSG_ADDLINE     "ADDLINE"
#define TOK_ADDLINE     "z"
#define MSG_SVSADDLINE	"SVSADDLINE"
#define TOK_SVSADDLINE	"SD"
#define MSG_GLINE	"GLINE"
#define TOK_GLINE	"}"
#define MSG_SJOIN	"SJOIN"
#define TOK_SJOIN	"~"
#define MSG_SETHOST 	"SETHOST"
#define TOK_SETHOST 	"AA"
#define MSG_SETIDENT 	"SETIDENT"
#define	TOK_SETIDENT	"AD"
#define MSG_SETNAME	"SETNAME"
#define TOK_SETNAME	"AE"
#define MSG_LAG		"LAG"
#define TOK_LAG		"AF"
#define MSG_SDESC       "SDESC"
#define TOK_SDESC       "AG"
#define MSG_STATSERV	"STATSERV"
#define TOK_STATSERV	"AH"
#define MSG_KNOCK	"KNOCK"
#define TOK_KNOCK	"AI"
#define MSG_CREDITS 	"CREDITS"
#define TOK_CREDITS 	"AJ"
#define MSG_LICENSE 	"LICENSE"
#define TOK_LICENSE 	"AK"
#define MSG_CHGHOST 	"CHGHOST"
#define TOK_CHGHOST 	"AL"
#define MSG_RPING   	"RPING"
#define TOK_RPING	"AM"
#define MSG_RPONG   	"RPONG"
#define TOK_RPONG	"AN"
#define MSG_NETINFO 	"NETINFO"
#define TOK_NETINFO 	"AO"
#define MSG_SENDUMODE 	"SENDUMODE"
#define TOK_SENDUMODE 	"AP"
#define MSG_ADDMOTD 	"ADDMOTD"
#define TOK_ADDMOTD	"AQ"
#define MSG_ADDOMOTD	"ADDOMOTD"
#define TOK_ADDOMOTD	"AR"
#define MSG_SVSMOTD	"SVSMOTD"
#define TOK_SVSMOTD	"AS"
#define MSG_SMO 	"SMO"
#define TOK_SMO 	"AU"
#define MSG_OPERMOTD 	"OPERMOTD"
#define TOK_OPERMOTD 	"OM"
#define MSG_LOCOPMOTD 	"LOCOPMOTD"
#define TOK_LOCOPMOTD 	"LM"
#define MSG_HELPERMOTD 	"HELPERMOTD"
#define TOK_HELPERMOTD 	"HM"
#define MSG_VIRUSMOTD 	"VIRUSMOTD"
#define TOK_VIRUSMOTD 	"VM"
#define MSG_TSCTL 	"TSCTL"
#define TOK_TSCTL 	"AW"
#define MSG_SVSJOIN 	"SVSJOIN"
#define TOK_SVSJOIN 	"AX"
#define MSG_SAJOIN 	"SAJOIN"
#define TOK_SAJOIN 	"AY"
#define MSG_SVSPART 	"SVSPART"
#define TOK_SVSPART 	"AX"
#define MSG_SAPART 	"SAPART"
#define TOK_SAPART 	"AY"
#define MSG_CHGIDENT 	"CHGIDENT"
#define TOK_CHGIDENT 	"AZ"
#define MSG_SWHOIS 	"SWHOIS"
#define TOK_SWHOIS 	"BA"
#define MSG_SVSO 	"SVSO"
#define TOK_SVSO 	"BB"
#define MSG_SVSFLINE 	"SVSFLINE"
#define TOK_SVSFLINE 	"BC"
#define MSG_TKL		"TKL"
#define TOK_TKL 	"BD"
#define MSG_VHOST 	"VHOST"
#define TOK_VHOST 	"BE"
#define MSG_BOTMOTD 	"BOTMOTD"
#define TOK_BOTMOTD 	"BF"
#define MSG_REMGLINE	"REMGLINE"
#define TOK_REMGLINE	"BG"
#define MSG_HTM		"HTM"
#define TOK_HTM		"BH"
#define MSG_UMODE2	"UMODE2"
#define TOK_UMODE2	"|"
#define MSG_DCCDENY	"DCCDENY"
#define TOK_DCCDENY	"BI"
#define MSG_UNDCCDENY   "UNDCCDENY"
#define TOK_UNDCCDENY   "BJ"
#define MSG_CHGNAME	"CHGNAME"
#define MSG_SVSNAME	"SVSNAME"
#define TOK_CHGNAME	"BK"
#define MSG_SHUN	"SHUN"
#define TOK_SHUN	"BL"
#define MSG_NEWJOIN 	"NEWJOIN"	/* For CR Java Chat */
#define MSG_POST	"POST"
#define TOK_POST	"BN"
#define MSG_INFOSERV 	"INFOSERV"
#define MSG_IS		"IS"
#define TOK_INFOSERV	"BO"
#define MSG_BOTSERV	"BOTSERV"
#define TOK_BOTSERV	"BS"
#define MAXPARA    	15

extern int m_private(), m_topic(), m_join(), m_part(), m_mode(), m_svsmode();
extern int m_ping(), m_pong(), m_wallops(), m_kick(), m_svsnick();
extern int m_nick(), m_error(), m_notice(), m_samode(), m_svsnoop();
extern int m_invite(), m_quit(), m_kill(), m_svskill(), m_identify(), m_auth();
extern int m_akill(), m_kline(), m_unkline(), m_rakill(), m_sqline();
extern int m_zline(), m_unzline();
extern int m_gnotice(), m_goper(), m_globops(), m_locops(), m_helpers(), m_virus(), m_unsqline(), m_chatops();
extern int m_protoctl();
extern int m_motd(), m_who(), m_whois(), m_user(), m_list();
extern int m_server(), m_info(), m_links(), m_linkinfo(), m_summon(), m_stats();
extern int m_users(), m_version(), m_help();
extern int m_services(), m_backserv();
extern int m_statserv(), m_botserv(), m_statserv();
extern int m_squit(), m_away(), m_connect();
extern int m_oper(), m_pass(), m_trace();
extern int m_time(), m_names(), m_admin();
extern int m_lusers(), m_umode(), m_close();
extern int m_motd(), m_whowas(), m_silence();
extern int m_service(), m_userhost(), m_ison(), m_watch();
extern int m_service(), m_servset(), m_servlist(), m_squery();
extern int m_rehash(), m_restart(), m_die(), m_dns(), m_hash();
/*extern int m_noshortn(),m_noshortc(),m_noshortm(),m_noshorto(),m_noshorth();*/

extern int m_gline(), m_remgline(), m_map(), m_svs2mode(), m_dalinfo();
extern int m_addline(), m_svsaddline(), m_rules(), m_mkpasswd();
extern int m_sethost(), m_setident(), m_setname();
extern int m_lag(), m_sdesc(), m_knock(), m_credits();
extern int m_license(), m_chghost(), m_rping(), m_rpong();
extern int m_netinfo(), m_sendumode(), m_addmotd(), m_addomotd();
extern int m_svsmotd(), m_opermotd(), m_locopmotd(), m_helpermotd(), m_virusmotd(), m_tsctl();
extern int m_svsjoin(), m_sajoin(), m_svspart(), m_sapart();
extern int m_chgident(), m_swhois(), m_svso(), m_svsfline();
extern int m_tkl(), m_vhost(), m_botmotd(), m_sjoin(), m_htm();
extern int m_umode2(), m_dccdeny(), m_undccdeny();
extern int m_chgname(), m_shun(), m_post();
#ifdef GUEST
extern int m_guest();
#endif

#ifdef MSGTAB
struct Message *msgmap[256];
struct Message msgtab[] = {
	{MSG_PRIVATE, m_private, 0, MAXPARA, TOK_PRIVATE, 0L},
	{MSG_NOTICE, m_notice, 0, MAXPARA, TOK_NOTICE, 0L},
	{MSG_MODE, m_mode, 0, MAXPARA, TOK_MODE, 0L},
	{MSG_NICK, m_nick, 0, MAXPARA, TOK_NICK, 0L},
	{MSG_JOIN, m_join, 0, MAXPARA, TOK_JOIN, 0L},
	{MSG_PING, m_ping, 0, MAXPARA, TOK_PING, 0L},
	{MSG_WHOIS, m_whois, 0, MAXPARA, TOK_WHOIS, 0L},
	{MSG_ISON, m_ison, 0, 1, TOK_ISON, 0L},
	{MSG_USER, m_user, 0, MAXPARA, TOK_USER, 0L},
	{MSG_PONG, m_pong, 0, MAXPARA, TOK_PONG, 0L},
	{MSG_PART, m_part, 0, MAXPARA, TOK_PART, 0L},
	{MSG_QUIT, m_quit, 0, MAXPARA, TOK_QUIT, 0L},
	{MSG_WATCH, m_watch, 0, 1, TOK_WATCH, 0L},
	{MSG_USERHOST, m_userhost, 0, 1, TOK_USERHOST, 0L},
	{MSG_SVSNICK, m_svsnick, 0, MAXPARA, TOK_SVSNICK, 0L},
	{MSG_SVSMODE, m_svsmode, 0, MAXPARA, TOK_SVSMODE, 0L},
	{MSG_LUSERS, m_lusers, 0, MAXPARA, TOK_LUSERS, 0L},
	{MSG_IDENTIFY, m_identify, 0, 1, TOK_IDENTIFY, 0L},
	{MSG_AUTH, m_auth, 0, 1, TOK_AUTH, 0L},
	{MSG_CHANSERV, m_services, 0, 1, TOK_CHANSERV, 0L},
	{MSG_TOPIC, m_topic, 0, MAXPARA, TOK_TOPIC, 0L},
	{MSG_INVITE, m_invite, 0, MAXPARA, TOK_INVITE, 0L},
	{MSG_KICK, m_kick, 0, MAXPARA, TOK_KICK, 0L},
	{MSG_WALLOPS, m_wallops, 0, 1, TOK_WALLOPS, 0L},
	{MSG_ERROR, m_error, 0, MAXPARA, TOK_ERROR, 0L},
	{MSG_KILL, m_kill, 0, MAXPARA, TOK_KILL, 0L},
	{MSG_PROTOCTL, m_protoctl, 0, MAXPARA, TOK_PROTOCTL, 0L},
	{MSG_AWAY, m_away, 0, MAXPARA, TOK_AWAY, 0L},
	{MSG_SERVER, m_server, 0, MAXPARA, TOK_SERVER, 0L},
	{MSG_SQUIT, m_squit, 0, MAXPARA, TOK_SQUIT, 0L},
	{MSG_WHO, m_who, 0, MAXPARA, TOK_WHO, 0L},
	{MSG_WHOWAS, m_whowas, 0, MAXPARA, TOK_WHOWAS, 0L},
	{MSG_LIST, m_list, 0, MAXPARA, TOK_LIST, 0L},
	{MSG_NAMES, m_names, 0, MAXPARA, TOK_NAMES, 0L},
	{MSG_TRACE, m_trace, 0, MAXPARA, TOK_TRACE, 0L},
	{MSG_PASS, m_pass, 0, MAXPARA, TOK_PASS, 0L},
	{MSG_TIME, m_time, 0, MAXPARA, TOK_TIME, 0L},
	{MSG_OPER, m_oper, 0, MAXPARA, TOK_OPER, 0L},
	{MSG_CONNECT, m_connect, 0, MAXPARA, TOK_CONNECT, 0L},
	{MSG_VERSION, m_version, 0, MAXPARA, TOK_VERSION, 0L},
	{MSG_STATS, m_stats, 0, MAXPARA, TOK_STATS, 0L},
	{MSG_LINKS, m_links, 0, MAXPARA, TOK_LINKS, 0L},
	{MSG_LINKINFO, m_linkinfo, 0, MAXPARA, TOK_LINKINFO, 0L},
	{MSG_ADMIN, m_admin, 0, MAXPARA, TOK_ADMIN, 0L},
	{MSG_SUMMON, m_summon, 0, 1, TOK_SUMMON, 0L},
	{MSG_USERS, m_users, 0, MAXPARA, TOK_USERS, 0L},
	{MSG_SAMODE, m_samode, 0, MAXPARA, TOK_SAMODE, 0L},
	{MSG_SVSKILL, m_svskill, 0, MAXPARA, TOK_SVSKILL, 0L},
	{MSG_SVSNOOP, m_svsnoop, 0, MAXPARA, TOK_SVSNOOP, 0L},
	{MSG_CS, m_services, 0, 1, TOK_CHANSERV, 0L},
	{MSG_NICKSERV, m_services, 0, 1, TOK_NICKSERV, 0L},
	{MSG_NS, m_services, 0, 1, TOK_NICKSERV, 0L},
	{MSG_INFOSERV, m_services, 0, 1, TOK_INFOSERV, 0L},
	{MSG_IS, m_services, 0, 1, TOK_INFOSERV, 0L},
	{MSG_BACKUP, m_backserv, 0, 1, TOK_BACKUP, 0L},
	{MSG_BU, m_backserv, 0, 1, TOK_BACKUP, 0L},
	{MSG_MEMOSERV, m_services, 0, 1, TOK_MEMOSERV, 0L},
	{MSG_MS, m_services, 0, 1, TOK_MEMOSERV, 0L},
	{MSG_HELPSERV, m_services, 0, 1, TOK_HELPSERV, 0L},
	{MSG_HS, m_services, 0, 1, TOK_HELPSERV, 0L},
	{MSG_SERVICES, m_services, 0, 1, TOK_SERVICES, 0L},
	{MSG_SS, m_services, 0, 1, TOK_SERVICES, 0L},
	{MSG_HELP, m_help, 0, 1, TOK_HELP, 0L},
	{MSG_HELPOP, m_help, 0, 1, TOK_HELP, 0L},
	{MSG_INFO, m_info, 0, MAXPARA, TOK_INFO, 0L},
	{MSG_MOTD, m_motd, 0, MAXPARA, TOK_MOTD, 0L},
	{MSG_CLOSE, m_close, 0, MAXPARA, TOK_CLOSE, 0L},
	{MSG_SILENCE, m_silence, 0, MAXPARA, TOK_SILENCE, 0L},
	{MSG_AKILL, m_akill, 0, MAXPARA, TOK_AKILL, 0L},
	{MSG_SQLINE, m_sqline, 0, MAXPARA, TOK_SQLINE, 0L},
	{MSG_UNSQLINE, m_unsqline, 0, MAXPARA, TOK_UNSQLINE, 0L},
	{MSG_KLINE, m_kline, 0, MAXPARA, TOK_KLINE, 0L},
	{MSG_UNKLINE, m_unkline, 0, MAXPARA, TOK_UNKLINE, 0L},
	{MSG_ZLINE, m_zline, 0, MAXPARA, TOK_ZLINE, 0L},
	{MSG_UNZLINE, m_unzline, 0, MAXPARA, TOK_UNZLINE, 0L},
	{MSG_RAKILL, m_rakill, 0, MAXPARA, TOK_RAKILL, 0L},
	{MSG_GNOTICE, m_gnotice, 0, MAXPARA, TOK_GNOTICE, 0L},
	{MSG_GOPER, m_goper, 0, MAXPARA, TOK_GOPER, 0L},
	{MSG_GLOBOPS, m_globops, 0, MAXPARA, TOK_GLOBOPS, 0L},
	{MSG_HELPERS, m_helpers, 0, MAXPARA, TOK_HELPERS, 0L},
	{MSG_VIRUS, m_virus, 0, MAXPARA, TOK_VIRUS, 0L},
	{MSG_CHATOPS, m_chatops, 0, 1, TOK_CHATOPS, 0L},
	{MSG_LOCOPS, m_locops, 0, 1, TOK_LOCOPS, 0L},
	{MSG_HASH, m_hash, 0, MAXPARA, TOK_HASH, 0L},
	{MSG_DNS, m_dns, 0, MAXPARA, TOK_DNS, 0L},
	{MSG_REHASH, m_rehash, 0, MAXPARA, TOK_REHASH, 0L},
	{MSG_RESTART, m_restart, 0, MAXPARA, TOK_RESTART, 0L},
	{MSG_DIE, m_die, 0, MAXPARA, TOK_DIE, 0L},
	{MSG_RULES, m_rules, 0, MAXPARA, TOK_RULES, 0L},
	{MSG_MAP, m_map, 0, MAXPARA, TOK_MAP, 0L},
	{MSG_GLINE, m_gline, 0, MAXPARA, TOK_GLINE, 0L},
	{MSG_REMGLINE, m_remgline, 0, MAXPARA, TOK_REMGLINE, 0L},
	{MSG_DALINFO, m_dalinfo, 0, MAXPARA, TOK_DALINFO, 0L},
	{MSG_SVS2MODE, m_svs2mode, 0, MAXPARA, TOK_SVS2MODE, 0L},
	{MSG_MKPASSWD, m_mkpasswd, 0, MAXPARA, TOK_MKPASSWD, 0L},
	{MSG_ADDLINE, m_addline, 0, 1, TOK_ADDLINE, 0L},
	{MSG_SVSADDLINE, m_svsaddline, 0, 1, TOK_SVSADDLINE, 0L},
	{MSG_SETHOST, m_sethost, 0, MAXPARA, TOK_SETHOST, 0L},
	{MSG_SETIDENT, m_setident, 0, MAXPARA, TOK_SETIDENT, 0L},
	{MSG_SETNAME, m_setname, 0, 1, TOK_SETNAME, 0L},
	{MSG_LAG, m_lag, 0, MAXPARA, TOK_LAG, 0L},
	{MSG_SDESC, m_sdesc, 0, 1, TOK_SDESC, 0L},
	{MSG_STATSERV, m_statserv, 0, 1, TOK_STATSERV, 0L},
	{MSG_KNOCK, m_knock, 0, 2, TOK_KNOCK, 0L},
	{MSG_CREDITS, m_credits, 0, MAXPARA, TOK_CREDITS, 0L},
	{MSG_LICENSE, m_license, 0, MAXPARA, TOK_LICENSE, 0L},
	{MSG_CHGHOST, m_chghost, 0, MAXPARA, TOK_CHGHOST, 0L},
	{MSG_RPING, m_rping, 0, MAXPARA, TOK_RPING, 0L},
	{MSG_RPONG, m_rpong, 0, MAXPARA, TOK_RPONG, 0L},
	{MSG_NETINFO, m_netinfo, 0, MAXPARA, TOK_NETINFO, 0L},
	{MSG_SENDUMODE, m_sendumode, 0, MAXPARA, TOK_SENDUMODE, 0L},
	{MSG_SMO, m_sendumode, 0, MAXPARA, TOK_SMO, 0L},
	{MSG_ADDMOTD, m_addmotd, 0, 1, TOK_ADDMOTD, 0L},
	{MSG_ADDOMOTD, m_addomotd, 0, 1, TOK_ADDOMOTD, 0L},
	{MSG_SVSMOTD, m_svsmotd, 0, MAXPARA, TOK_SVSMOTD, 0L},
	{MSG_OPERMOTD, m_opermotd, 0, MAXPARA, TOK_OPERMOTD, 0L},
	{MSG_LOCOPMOTD, m_locopmotd, 0, MAXPARA, TOK_LOCOPMOTD, 0L},
	{MSG_HELPERMOTD, m_helpermotd, 0, MAXPARA, TOK_HELPERMOTD, 0L},
	{MSG_VIRUSMOTD, m_virusmotd, 0, MAXPARA, TOK_VIRUSMOTD, 0L},
	{MSG_TSCTL, m_tsctl, 0, MAXPARA, TOK_TSCTL, 0L},
	{MSG_SVSJOIN, m_svsjoin, 0, MAXPARA, TOK_SVSJOIN, 0L},
	{MSG_SAJOIN, m_sajoin, 0, MAXPARA, TOK_SAJOIN, 0L},
	{MSG_SVSPART, m_svspart, 0, MAXPARA, TOK_SVSPART, 0L},
	{MSG_SAPART, m_sapart, 0, MAXPARA, TOK_SAPART, 0L},
	{MSG_CHGIDENT, m_chgident, 0, MAXPARA, TOK_CHGIDENT, 0L},
	{MSG_SWHOIS, m_swhois, 0, MAXPARA, TOK_SWHOIS, 0L},
	{MSG_SVSO, m_svso, 0, MAXPARA, TOK_SVSO, 0L},
	{MSG_SVSFLINE, m_svsfline, 0, MAXPARA, TOK_SVSFLINE, 0L},
	{MSG_TKL, m_tkl, 0, MAXPARA, TOK_TKL, 0L},
	{MSG_VHOST, m_vhost, 0, MAXPARA, TOK_VHOST, 0L},
	{MSG_BOTMOTD, m_botmotd, 0, MAXPARA, TOK_BOTMOTD, 0L},
	{MSG_SJOIN, m_sjoin, 0, MAXPARA, TOK_SJOIN, 0L},
	{MSG_HTM, m_htm, 0, MAXPARA, TOK_HTM, 0L},
	{MSG_UMODE2, m_umode2, 0, MAXPARA, TOK_UMODE2, 0L},
	{MSG_DCCDENY, m_dccdeny, 0, 2, TOK_DCCDENY, 0L},
	{MSG_UNDCCDENY, m_undccdeny, 0, MAXPARA, TOK_UNDCCDENY, 0L},
	{MSG_CHGNAME, m_chgname, 0, MAXPARA, TOK_CHGNAME, 0L},
	{MSG_SVSNAME, m_chgname, 0, MAXPARA, TOK_CHGNAME, 0L},
	{MSG_SHUN, m_shun, 0, MAXPARA, TOK_SHUN, 0L},
	{MSG_NEWJOIN, m_join, 0, MAXPARA, TOK_JOIN, 0L},
	{MSG_BOTSERV, m_botserv, 0, MAXPARA, TOK_BOTSERV,0L},
	{TOK_BOTSERV, m_botserv, 0, MAXPARA, TOK_BOTSERV,0L},
	{(char *)0, (int (*)())0, 0, 0, 0, 0L}
};

#else
extern struct Message msgtab[];
extern struct Message *msgmap[256];
#endif
#endif /* __msg_include__ */
