/************************************************************************
 *   Internet Relay Chat Daemon, src/dynconf.c
 *   (C) 1999-2000 Carsten Munk (Techie/Stskeeps) <stskeeps@tspre.org>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 1, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "struct.h"
#include "common.h"
#include "sys.h"
#include "h.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "userload.h"
#include "proto.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#ifdef _WIN32
#include <io.h>
#endif
#include <fcntl.h>
#include "h.h"
#include "setup.h"

ID_Copyright("(C) 2001 Angeles, AbyssNet Network Ltd");

#define DoDebug fprintf(stderr, "[%s] %s | %li\n", babuf, __FILE__, __LINE__);
#define AllocCpy(x,y) if ((x) && type == 1) MyFree((x)); x = (char *) MyMalloc(strlen(y) + 1); strcpy(x,y)
#define XtndCpy(x,y) x = (char *) MyMalloc(strlen(y) + 2); *x = '\0'; strcat(x, "*"); strcpy(x,y)

/* externals */
extern int un_uid, un_gid;
/* internals */
aConfiguration iConf;
int  icx = 0;
char buf[1024];

adConfItem *cdconf = NULL;


/* strips \r and \n's from the line */
void iCstrip(char *line)
{
	char *c;

	if ((c = strchr(line, '\n')))
		*c = '\0';
	if ((c = strchr(line, '\r')))
		*c = '\0';
}

/* this loads dynamic ZCONF */
int  load_conf(char *filename, int type)
{
	FILE *zConf;
	char *version = NULL;
	char *i;

	zConf = fopen(filename, "r");

	if (!zConf)
	{
		if (type == 1)
		{
			sendto_ops("[error] Could not load %s !", filename);
			return -1;
		}
		else
		{
#ifdef _WIN32
			ircsprintf(buf, "Unable to load dynamic config %s",
			    filename);
			MessageBox(NULL, "IRCD/32 Init Error", buf,
			    MB_OK);
#else
			fprintf(stderr, "[error] Couldn't load %s !!!\n",
			    filename);
#endif
			exit(-1);
		}
	}
	i = fgets(buf, 1023, zConf);
	if (!i)
	{
		if (type == 1)
		{
			sendto_ops("[error] Error reading from %s !", filename);
			return -1;
		}
		else
		{
#ifdef _WIN32
			ircsprintf(buf, "Unable to read dynamic config %s",
			    filename);
			MessageBox(NULL, "IRCD/32 Init Error", buf,
			    MB_OK);
#else
			fprintf(stderr, "[error] Couldn't read %s !!!\n",
			    filename);
#endif
			exit(-1);
		}
	}
	/* Don't want this changed.. moved here. */
	AllocCpy(hidden_host, "x");
	initdynconf(zConf, filename, type);
	return -1;
}

int  opendconf(char *occnf)
{
	return open(occnf, O_RDONLY);
}

/* Init Dynamic Configuration...
	From initconf..
	--Angeles
*/
int  initdynconf(FILE *nconf, char *filename, int type)
{
	static char quotes[9][2] = { {'b', '\b'}, {'f', '\f'}, {'n', '\n'},
	{'r', '\r'}, {'t', '\t'}, {'v', '\v'},
	{'\\', '\\'}, {0, 0}
	};
	char *tmp, *s;
	int  fd, i;
	int iscomment = 0;
	char line[512], c[80];
	adConfItem *dconf = NULL;

	Debug((DEBUG_DEBUG, "initdconf(): dynconf.conf = %s", nconf));
	if ((fd = opendconf(filename)) == -1)
	{
		return -1;
	}
	(void)dgets(-1, NULL, 0);	/* make sure buffer is at empty pos */
	while ((i = dgets(fd, line, sizeof(line) - 1)) > 0)
	{
		line[i] = '\0';
		iCstrip(line);
/*		while (dgets(fd, c, sizeof(c) - 1) > 0)
		{
		} */
		/*
		 * Do quoting of characters and # detection.
		 */
		for (tmp = line; *tmp; tmp++)
		{
			if ((*tmp == '/') && (iscomment < 1))
			{
			    tmp++;
			    if (*tmp == '*')
			    {
				iscomment = 1;
				break;
			    }
			    else
				tmp--;
			}
			else if ((*tmp == '*') && (iscomment > 0))
			{
			    tmp++;
			    if (*tmp == '/')
			    {
				iscomment = 0;
				break;
			    }
			    else
				tmp--;
			}
			if (*tmp == '\\')
			{
				for (i = 0; quotes[i][0]; i++)
					if (quotes[i][0] == *(tmp + 1))
					{
						*tmp = quotes[i][1];
						break;
					}
				if (!quotes[i][0])
					*tmp = *(tmp + 1);
				if (!*(tmp + 1))
					break;
				else
					for (s = tmp; *s = *(s + 1); s++)
						;
			}
		}
		if (!*line || line[0] == '#' || line[0] == '\n' 
		    || line[0] == ' ' || line[0] == '\t' || iscomment > 0)
			continue;

		/* Could we test if it's conf line at all?      -Vesa */
		if (line[1] != ':')
		{
			Debug((DEBUG_ERROR, "Bad config line: %s", line));
			continue;
		}
		if (dconf)
			free_dconf(dconf);
		dconf = make_dconf();

		tmp = getfield(line);

		if (!tmp)
			continue;
		switch (*tmp)
		{
		  case 'A':	/* About */
		  case 'a':
			  dconf->status = DCONF_ABOUT;
			  break;
		  case 'C':	/* Some Misc Concept Stuff */
		  case 'c':
		  	  dconf->status = DCONF_CNCPT;
			  break;
		  case 'H':	/* Hosts */
		  case 'h':
		  	  dconf->status = DCONF_HOSTS;
			  break;
		  case 'M':	/* Some Misc Concept Stuff */
		  case 'm':
		  	  dconf->status = DCONF_MODE;
			  break;
		  case 'N':	/* Things for the  network */
		  case 'n':
		  	  dconf->status = DCONF_NETSET;
			  break;
		  case 'R':	/* Restriction Thingies */
		  case 'r':
		  	  dconf->status = DCONF_RESTRICT;
			  break;
		  case 'U':	/* Handle those Users */
		  case 'u':
		  	  dconf->status = DCONF_USER;
			  break;
		  default:
			  Debug((DEBUG_ERROR, "Error in config file: %s",
			      line));
			  break;
		}
		if (dIsIllegal(dconf))
		{
			continue;
		}
		for (;;)	
		{
		/* Fake loop, that I can use break here --Angeles [Shown to me by msa] */
			if ((tmp = getfield(NULL)) == NULL)
				break;
			DupString(dconf->c_1, tmp);
			if ((tmp = getfield(NULL)) == NULL)
				break;
			DupString(dconf->c_2, tmp);
			if ((tmp = getfield(NULL)) == NULL)
				break;
			DupString(dconf->c_3, tmp);
			if ((tmp = getfield(NULL)) == NULL)
				break;
			DupString(dconf->c_4, tmp);
			if ((tmp = getfield(NULL)) == NULL)
				break;
			DupString(dconf->c_5, tmp);
		}
		if (dIsIllegal(dconf))
		{
			continue;
		}
		if (dconf->status & DCONF_MODE) {
		    switch (*dconf->c_1)
		    {
			case 'x':
			    MODE_X = atoi(dconf->c_2);
			    break;
			case 'G':
			    MODE_STRIPWORDS = atoi(dconf->c_2);
			    break;
			case 'i':
			    MODE_I = atoi(dconf->c_2);
			    break;
			case 'w':
			    MODE_W = atoi(dconf->c_2);
			    break;
			case 'W':
			    MODE_WI = atoi(dconf->c_2);
			    break;
			default:
			    break;
		    }
		}
		else if (dconf->status & DCONF_NETSET) {
		    TRUEHUB = atoi(dconf->c_1);
#ifndef HUB
    		    if ((TRUEHUB == 1) && (type == 0))
		    {
#ifdef _WIN32
			MessageBox(NULL,
			    "IRCd/32 Init Error",
			    "TRUEHUB value set to 1, but I'm not a hub!",
			    MB_OK);
#else
			fprintf(stderr,
			    "[error] I'm a leaf NOT a hub!\n");
#endif
			exit(-1);
		    }
		    else if ((type == 1) && (TRUEHUB == 1))
		    {
			sendto_ops
			    ("TRUEHUB value set to 1, but I'm not a hub!");
		    }
#endif
		    ALLOW_CHATOPS = atoi(dconf->c_2);
		    WEBTV_SUPPORT = atoi(dconf->c_3);
		    HIDE_ULINES = atoi(dconf->c_4);
		    NO_OPER_HIDING = atoi(dconf->c_5);
		}
		else if (dconf->status & DCONF_USER) {
		    SHOWOPERS = atoi(dconf->c_1);
		    SHOWOPERMOTD = atoi(dconf->c_2);
		    MAXCHANNELSPERUSER = atol(dconf->c_3);
		    AllocCpy(AUTO_JOIN_CHANS, dconf->c_4);
		    AllocCpy(OPER_AUTO_JOIN_CHANS, dconf->c_5);
		}
		else if (dconf->status & DCONF_RESTRICT) {
		    iConf.socksbantime = atol(dconf->c_1);
		    KILLDIFF = atoi(dconf->c_2);
		    AllocCpy(KLINE_ADDRESS, dconf->c_3);
		    AllocCpy(iConf.socksbanmessage, dconf->c_4);
		    AllocCpy(iConf.socksquitmessage, dconf->c_5);
		}
		else if (dconf->status & DCONF_CNCPT) {
		    HOST_TIMEOUT = atol(dconf->c_1);
		    HOST_RETRIES = atoi(dconf->c_2);
		    iNAH = atoi(dconf->c_3);
#ifndef BIG_SECURITY_HOLE
		    un_uid = atoi(dconf->c_4);
		    un_gid = atoi(dconf->c_5);
#endif
		}
		else if (dconf->status & DCONF_ABOUT) {
		    AllocCpy(ircnetwork, dconf->c_1);
		    AllocCpy(defserv, dconf->c_2);
		    AllocCpy(SERVICES_NAME, dconf->c_3);
		    AllocCpy(STATS_SERVER, dconf->c_4);
		    AllocCpy(helpchan, dconf->c_5);
		}
		else if (dconf->status & DCONF_HOSTS) {
		    AllocCpy(oper_host, dconf->c_1);
		    AllocCpy(locop_host, dconf->c_2);
		    AllocCpy(virii_host, dconf->c_3);
		    AllocCpy(helper_host, dconf->c_4);
		    AllocCpy(netdomain, dconf->c_5);
		}
		Debug((DEBUG_NOTICE,
		    "Read DConf: (%s) (%s) (%s) (%s) [%d]",
		    dconf->c_1, dconf->c_2, dconf->c_3,
		    dconf->c_3, dconf->status));
		dconf->next = cdconf;
		cdconf = dconf;
		dconf = NULL;
	}
	if (dconf)
		free_dconf(dconf);
	(void)dgets(-1, NULL, 0);	/* make sure buffer is at empty pos */
	(void)close(fd);
	if (type == 0)
	{
		fprintf(stderr, "* Loaded %s ..\n", filename);
	}
	else
	{
		sendto_realops("Loaded %s ..", filename);
	}
	return 0;
}
/* Make this to it sets defaults, instead of complaining like a little
    bitch >:)
	--Angeles
*/
void doneconf(int type)
{
        /* calculate on errors */
        char *errormsg = MyMalloc(16384);

        *errormsg = '\0';
        if (!KLINE_ADDRESS || (*KLINE_ADDRESS == '\0'))
                strcat(errormsg, "- Missing KLINE_ADDRESS\n");
#ifndef DEVELOP
        if (KLINE_ADDRESS)
                if (!strchr(KLINE_ADDRESS, '@') && !strchr(KLINE_ADDRESS, ':'))
                {
                        strcat(errormsg,
                            "- KLINE_ADDRESS is not an e-mail or a URL\n");
                }
#endif
        if ((MAXCHANNELSPERUSER < 1))
                strcat(errormsg,
                    "- MAXCHANNELSPERUSER is an invalid value, must be > 0\n");
        if ((iNAH < 0) || (iNAH > 1))
                strcat(errormsg, "- iNAH is an invalid value\n");
        if (AUTO_JOIN_CHANS == '\0')
                strcat(errormsg, "- AUTO_JOIN_CHANS is an invalid value\n");
        if (OPER_AUTO_JOIN_CHANS == '\0')
                strcat(errormsg,
                    "- OPER_AUTO_JOIN_CHANS is an invalid value\n");
        if (HOST_TIMEOUT < 0 || HOST_TIMEOUT > 180)
                strcat(errormsg, "- HOST_TIMEOUT is an invalid value\n");
        if (HOST_RETRIES < 0 || HOST_RETRIES > 10)
                strcat(errormsg, "- HOST_RETRIES is an invalid value\n");
#define Missing(x) !(x) || (*(x) == '\0')
        if (Missing(defserv))
                strcat(errormsg, "- Missing defserv field\n");
        if (Missing(SERVICES_NAME))
                strcat(errormsg, "- Missing SERVICES_NAME field\n");
        if (Missing(oper_host))
                strcat(errormsg, "- Missing oper_host field\n");
        if (Missing(locop_host))
                strcat(errormsg, "- Missing locop_host field\n");
        if (Missing(virii_host))
                strcat(errormsg, "- Missing virii_host field\n");
        if (Missing(helper_host))
                strcat(errormsg, "- Missing helper_host field\n");
        if (Missing(hidden_host))
                strcat(errormsg, "- Missing hidden_host field\n");
        if (Missing(netdomain))
                strcat(errormsg, "- Missing netdomain field\n");
        if (Missing(helpchan))
                strcat(errormsg, "- Missing helpchan field\n");
        if (Missing(STATS_SERVER))
                strcat(errormsg, "- Missing STATS_SERVER field\n");
        if (Missing(iConf.socksbanmessage))
                strcat(errormsg, "- Missing SOCKSBANMESSAGE field\n");
        if (Missing(iConf.socksquitmessage))
                strcat(errormsg, "- Missing SOCKSQUITMESSAGE field\n");
        if (*errormsg != '\0')
        {
                fprintf(stderr, "\n*** ERRORS IN DYNAMIC CONFIGURATION\n");
		fprintf(stderr, errormsg);
                MyFree(errormsg);
                exit(-1);
        }
        MyFree(errormsg);

}

void init_dynconf(void)
{
	bzero(&iConf, sizeof(iConf));
}

/* Report the dyna.conf info -codemastr*/
void report_dynconf(aClient *sptr)
{
	sendto_one(sptr, ":%s %i %s :\2Dynamic Configuration Report\2",
	    me.name, RPL_TEXT, sptr->name);
	sendto_one(sptr, ":%s %i %s :KLINE_ADDRESS       : %s", me.name, RPL_TEXT,
	    sptr->name, KLINE_ADDRESS);
	sendto_one(sptr, ":%s %i %s :MODE_X              : %d", me.name, RPL_TEXT, sptr->name,
	    MODE_X);
	sendto_one(sptr, ":%s %i %s :MODE_STRIPWORDS     : %d", me.name, RPL_TEXT,
	    sptr->name, MODE_STRIPWORDS);
	sendto_one(sptr, ":%s %i %s :MODE_I              : %d", me.name, RPL_TEXT, sptr->name,
	    MODE_I);
	sendto_one(sptr, ":%s %i %s :TRUEHUB             : %d", me.name, RPL_TEXT,
	    sptr->name, TRUEHUB);
	sendto_one(sptr, ":%s %i %s :SHOWOPERS           : %d", me.name, RPL_TEXT,
	    sptr->name, SHOWOPERS);
	sendto_one(sptr, ":%s %i %s :KILLDIFF            : %d", me.name, RPL_TEXT,
	    sptr->name, KILLDIFF);
	sendto_one(sptr, ":%s %i %s :SHOWOPERMOTD        : %d", me.name, RPL_TEXT,
	    sptr->name, SHOWOPERMOTD);
	sendto_one(sptr, ":%s %i %s :HIDE_ULINES         : %d", me.name, RPL_TEXT,
	    sptr->name, HIDE_ULINES);
	sendto_one(sptr, ":%s %i %s :ALLOW_CHATOPS       : %d", me.name, RPL_TEXT,
	    sptr->name, ALLOW_CHATOPS);
	sendto_one(sptr, ":%s %i %s :SOCKS_BAN_MESSAGE   : %s", me.name, RPL_TEXT,
	    sptr->name, iConf.socksbanmessage);
	sendto_one(sptr, ":%s %i %s :SOCKS_QUIT_MESSAGE  : %s", me.name, RPL_TEXT,
	    sptr->name, iConf.socksquitmessage);
	sendto_one(sptr, ":%s %i %s :SOCKSBANTIME        : %i", me.name, RPL_TEXT,
	    sptr->name, iConf.socksbantime);
	sendto_one(sptr, ":%s %i %s :MAXCHANNELSPERUSER  : %i", me.name, RPL_TEXT,
	    sptr->name, MAXCHANNELSPERUSER);
	sendto_one(sptr, ":%s %i %s :WEBTV_SUPPORT       : %d", me.name, RPL_TEXT,
	    sptr->name, WEBTV_SUPPORT);
	sendto_one(sptr, ":%s %i %s :NO_OPER_HIDING      : %d", me.name, RPL_TEXT,
	    sptr->name, NO_OPER_HIDING);
	sendto_one(sptr, ":%s %i %s :AUTO_JOIN_CHANS     : %s", me.name, RPL_TEXT,
	    sptr->name, AUTO_JOIN_CHANS);
	sendto_one(sptr, ":%s %i %s :OPER_AUTO_JOIN_CHANS: %s", me.name,
	    RPL_TEXT, sptr->name, OPER_AUTO_JOIN_CHANS);
	sendto_one(sptr, ":%s %i %s :HOST_TIMEOUT        : %li", me.name, RPL_TEXT,
	    sptr->name, HOST_TIMEOUT);
	sendto_one(sptr, ":%s %i %s :HOST_RETRIES        : %d", me.name, RPL_TEXT,
	    sptr->name, HOST_RETRIES);
	sendto_one(sptr, ":%s %i %s :\2End Of Report\2",
	    me.name, RPL_TEXT, sptr->name);
}

/* Report the network file info -codemastr */
void report_network(aClient *sptr)
{
	sendto_one(sptr, ":%s %i %s :\2Network Configuration Report\2",
	    me.name, RPL_TEXT, sptr->name);
	sendto_one(sptr, ":%s %i %s :NETWORK       : %s", me.name, RPL_TEXT,
	    sptr->name, ircnetwork);
	sendto_one(sptr, ":%s %i %s :DEFAULT_SERVER: %s", me.name, RPL_TEXT,
	    sptr->name, defserv);
	sendto_one(sptr, ":%s %i %s :SERVICES_NAME : %s", me.name, RPL_TEXT,
	    sptr->name, SERVICES_NAME);
	sendto_one(sptr, ":%s %i %s :OPER_HOST     : %s", me.name, RPL_TEXT,
	    sptr->name, oper_host);
	sendto_one(sptr, ":%s %i %s :LOCOP_HOST    : %s", me.name, RPL_TEXT,
	    sptr->name, locop_host);
	sendto_one(sptr, ":%s %i %s :VIRII_HOST    : %s", me.name, RPL_TEXT,
	    sptr->name, virii_host);
	sendto_one(sptr, ":%s %i %s :HELPER_HOST   : %s", me.name, RPL_TEXT,
	    sptr->name, helper_host);
#ifdef DISABLE_USERMOD
	sendto_one(sptr, ":%s %i %s :                Note: DISABLE_USERMOD is defined", 
	    me.name, RPL_TEXT, sptr->name);
#endif
	sendto_one(sptr, ":%s %i %s :HIDDEN_HOST   : %s", me.name, RPL_TEXT,
	    sptr->name, hidden_host);
	sendto_one(sptr, ":%s %i %s :NETDOMAIN     : %s", me.name, RPL_TEXT,
	    sptr->name, netdomain);
	sendto_one(sptr, ":%s %i %s :HELPCHAN      : %s", me.name, RPL_TEXT,
	    sptr->name, helpchan);
	sendto_one(sptr, ":%s %i %s :STATS_SERVER  : %s", me.name, RPL_TEXT,
	    sptr->name, STATS_SERVER);
	sendto_one(sptr, ":%s %i %s :INAH          : %i", me.name, RPL_TEXT, sptr->name,
	    iNAH);
	sendto_one(sptr, ":%s %i %s :\2End Of Report\2",
	    me.name, RPL_TEXT, sptr->name);
}
